@@ -70,10 +70,11 @@ module C2HS.C.Trav (CT, readCT, transCT, runCT, throwCTExc, ifCTExc,
               -- C structure tree query functions
               --
               isTypedef, simplifyDecl, declrFromDecl, declrNamed,
-              declaredDeclr, declaredName, structMembers, expandDecl,
-              structName, enumName, tagName, isPtrDeclr, dropPtrDeclr,
-              isPtrDecl, isFunDeclr, structFromDecl, funResultAndArgs,
-              chaseDecl, findAndChaseDecl, checkForAlias, checkForOneCUName,
+              declaredDeclr, initDeclr, declaredName, structMembers, expandDecl,
+              structName, enumName, tagName, isPtrDeclr, isArrDeclr,
+              dropPtrDeclr, isPtrDecl, isArrDecl, isFunDeclr, structFromDecl,
+              funResultAndArgs, chaseDecl, findAndChaseDecl,
+              findAndChaseDeclOrTag, checkForAlias, checkForOneCUName,
               checkForOneAliasName, lookupEnum, lookupStructUnion,
               lookupDeclOrTag)
 where
@@ -98,6 +99,7 @@ import C2HS.C.Attrs     (AttrC(..), enterNewRangeC, enterNewObjRangeC,
                    setDefOfIdentC, updDefOfIdentC, CObj(..), CTag(..),
                    CDef(..))
 
+
 -- the C traversal monad
 -- ---------------------
 
@@ -160,7 +162,7 @@ runCT m ac s  = runCST m' (ac, s)
 ctExc :: String
 ctExc  = "ctExc"
 
--- | throw an exception 
+-- | throw an exception
 --
 throwCTExc :: CT s a
 throwCTExc  = throwExc ctExc "Error during traversal of a C structure tree"
@@ -316,7 +318,7 @@ getDefOf ide  = do
 -- | set the definition of an identifier
 --
 refersToDef         :: Ident -> CDef -> CT s ()
-refersToDef ide def  = 
+refersToDef ide def  =
   do traceCTrav $ "linking identifier: "++ dumpIdent ide ++ " --> " ++ show def
      transAttrCCT $ \akl -> (setDefOfIdentC akl ide def, ())
 
@@ -338,47 +340,55 @@ getDeclOf ide  =
       DontCareCD -> interr "CTrav.getDeclOf: Don't care!"
       TagCD _    -> interr "CTrav.getDeclOf: Illegal tag!"
       ObjCD obj  -> case obj of
-                      TypeCO    decl -> traceTypeCO decl >>
-                                        return decl
-                      ObjCO     decl -> traceObjCO decl >>
-                                        return decl
-                      EnumCO    _ _  -> illegalEnum
-                      BuiltinCO      -> illegalBuiltin
+                      TypeCO    decl        -> traceTypeCO decl >>
+                                               return decl
+                      ObjCO     decl        -> traceObjCO decl >>
+                                               return decl
+                      EnumCO    _ _         -> illegalEnum
+                      BuiltinCO Nothing     -> illegalBuiltin
+                      BuiltinCO (Just decl) -> traceBuiltinCO >>
+                                               return decl
   where
-    illegalEnum    = interr "CTrav.getDeclOf: Illegal enum!"
-    illegalBuiltin = interr "CTrav.getDeclOf: Attempted to get declarator of \
-                            \builtin entity!"
+    illegalEnum      = interr "CTrav.getDeclOf: Illegal enum!"
+    illegalBuiltin   = interr "CTrav.getDeclOf: Attempted to get declarator of \
+                              \builtin entity!"
                      -- if the latter ever becomes necessary, we have to
                      -- change the representation of builtins and give them
                      -- some dummy declarator
-    traceEnter  = traceCTrav $
-                    "Entering `getDeclOf' for `" ++ identToString ide
+    traceEnter       = traceCTrav
+                     $ "Entering `getDeclOf' for `" ++ identToString ide
                     ++ "'...\n"
-    traceTypeCO decl = traceCTrav $
-                    "...found a type object.\n" ++ show decl ++ "\n"
-    traceObjCO decl = traceCTrav $
-                    "...found a vanilla object.\n" ++ show decl ++ "\n"
-
+    traceTypeCO decl = traceCTrav
+                     $ "...found a type object:\n" ++ show decl ++ "\n"
+    traceObjCO decl  = traceCTrav
+                     $ "...found a vanilla object:\n" ++ show decl ++ "\n"
+    traceBuiltinCO   = traceCTrav
+                     $ "...found a builtin object with a proxy decl.\n"
 
 -- convenience functions
 --
 
+findTypeObjMaybeWith :: Bool -> Ident -> Bool -> CT s (Maybe (CObj, Ident))
+findTypeObjMaybeWith soft ide useShadows  =
+  do
+    oobj <- if useShadows
+            then findObjShadow ide
+            else liftM (fmap (\obj -> (obj, ide))) $ findObj ide
+    case oobj of
+      Just obj@(TypeCO _ ,   _) -> return $ Just obj
+      Just obj@(BuiltinCO _, _) -> return $ Just obj
+      Just _                    -> if soft
+                                   then return Nothing
+                                   else typedefExpectedErr ide
+      Nothing                   -> return $ Nothing
+
 -- | find a type object in the object name space; returns 'Nothing' if the
 -- identifier is not defined
 --
 -- * if the second argument is 'True', use 'findObjShadow'
 --
-findTypeObjMaybe                :: Ident -> Bool -> CT s (Maybe (CObj, Ident))
-findTypeObjMaybe ide useShadows  =
-  do
-    oobj <- if useShadows
-            then findObjShadow ide
-            else liftM (fmap (\obj -> (obj, ide))) $ findObj ide
-    case oobj of
-      Just obj@(TypeCO _ , _) -> return $ Just obj
-      Just obj@(BuiltinCO, _) -> return $ Just obj
-      Just _                  -> typedefExpectedErr ide
-      Nothing                 -> return $ Nothing
+findTypeObjMaybe :: Ident -> Bool -> CT s (Maybe (CObj, Ident))
+findTypeObjMaybe = findTypeObjMaybeWith False
 
 -- | find a type object in the object name space; raises an error and exception
 -- if the identifier is not defined
@@ -476,6 +486,15 @@ declaredDeclr decl                          =
   interr $ "CTrav.declaredDeclr: Too many declarators!\n\
            \  Declaration at " ++ show (posOf decl)
 
+-- | get the initialiser of a declaration that has at most one initialiser
+--
+initDeclr                            :: CDecl -> Maybe (CInitializer NodeInfo)
+initDeclr (CDecl _ []            _)  = Nothing
+initDeclr (CDecl _ [(_, ini, _)] _)  = ini
+initDeclr decl                          =
+  interr $ "CTrav.initDeclr: Too many declarators!\n\
+           \  Declaration at " ++ show (posOf decl)
+
 -- | get the name declared by a declaration that has exactly one declarator
 --
 declaredName      :: CDecl -> Maybe Ident
@@ -527,6 +546,17 @@ isPtrDeclr (CDeclr _ (CPtrDeclr _ _:_) _ _ _) = True
 isPtrDeclr (CDeclr _ (CArrDeclr _ _ _:_) _ _ _) = True
 isPtrDeclr _ = False
 
+-- | Need to distinguish between pointer and array declarations within
+-- structures.
+--
+isArrDeclr                                 :: CDeclr -> Maybe Int
+isArrDeclr (CDeclr _ (CArrDeclr _ sz _:_) _ _ _) = Just $ szToInt sz
+  where szToInt (CArrSize _ (CConst (CIntConst s _))) =
+          fromIntegral $ getCInteger s
+        szToInt _ = 1
+isArrDeclr _ = Nothing
+
+
 -- | drops the first pointer level from the given declarator
 --
 -- * the declarator must declare a pointer object
@@ -538,12 +568,12 @@ isPtrDeclr _ = False
 --        into an anonymous declarator and also change its attributes
 --
 dropPtrDeclr :: CDeclr -> CDeclr
-dropPtrDeclr (CDeclr ide (outermost:derived) asm ats node) = 
+dropPtrDeclr (CDeclr ide (outermost:derived) asm ats node) =
   case outermost of
     (CPtrDeclr _ _) -> CDeclr ide derived asm ats node
     (CArrDeclr _ _ _) -> CDeclr ide derived asm ats node
     _ -> interr "CTrav.dropPtrDeclr: No pointer!"
-  
+
 -- | checks whether the given declaration defines a pointer object
 --
 -- * there may only be a single declarator in the declaration
@@ -554,6 +584,12 @@ isPtrDecl (CDecl _ [(Just declr, _, _)] _)  = isPtrDeclr declr
 isPtrDecl _                                 =
   interr "CTrav.isPtrDecl: There was more than one declarator!"
 
+isArrDecl                                  :: CDecl -> Maybe Int
+isArrDecl (CDecl _ []                   _)  = Nothing
+isArrDecl (CDecl _ [(Just declr, _, _)] _)  = isArrDeclr declr
+isArrDecl _                                 =
+  interr "CTrav.isArrDecl: There was more than one declarator!"
+
 -- | checks whether the given declarator defines a function object
 --
 isFunDeclr                                 :: CDeclr -> Bool
@@ -590,7 +626,7 @@ funResultAndArgs cdecl@(CDecl specs [(Just declr, _, _)] _) =
   where
     funArgs (CDeclr _ide derived _asm _ats node) =
       case derived of
-        (CFunDeclr (Right (args,variadic)) _ats' _dnode : derived') -> 
+        (CFunDeclr (Right (args,variadic)) _ats' _dnode : derived') ->
           (args, CDeclr Nothing derived' Nothing [] node, variadic)
         (CFunDeclr (Left _) _ _ : _) ->
           interr "CTrav.funResultAndArgs: Old style function definition"
@@ -639,12 +675,34 @@ chaseDecl ide ind  =
 findAndChaseDecl                    :: Ident -> Bool -> Bool -> CT s CDecl
 findAndChaseDecl ide ind useShadows  =
   do
-    traceCTrav $ "findAndChaseDecl: " ++ show ide ++ "\n"
+    traceCTrav $ "findAndChaseDecl: " ++ show ide ++ " (" ++
+      show useShadows ++ ")\n"
     (obj, ide') <- findTypeObj ide useShadows   -- is there an object def?
     ide  `refersToNewDef` ObjCD obj
     ide' `refersToNewDef` ObjCD obj             -- assoc needed for chasing
     chaseDecl ide' ind
 
+findAndChaseDeclOrTag               :: Ident -> Bool -> Bool -> CT s CDecl
+findAndChaseDeclOrTag ide ind useShadows  =
+  do
+    traceCTrav $ "findAndChaseDeclOrTag: " ++ show ide ++ " (" ++
+      show useShadows ++ ")\n"
+    mobjide <- findTypeObjMaybeWith True ide useShadows -- is there an object def?
+    case mobjide of
+      Just (obj, ide') -> do
+        ide  `refersToNewDef` ObjCD obj
+        ide' `refersToNewDef` ObjCD obj             -- assoc needed for chasing
+        chaseDecl ide' ind
+      Nothing -> do
+        otag <- if useShadows
+                then findTagShadow ide
+                else liftM (fmap (\tag -> (tag, ide))) $ findTag ide
+        case otag of
+          Just (StructUnionCT su, _) -> do
+            let (CStruct _ _ _ _ nodeinfo) = su
+            return $ CDecl [CTypeSpec (CSUType su nodeinfo)] [] nodeinfo
+          _ -> unknownObjErr ide
+
 -- | given a declaration (which must have exactly one declarator), if the
 -- declarator is an alias, chase it to the actual declaration
 --
@@ -664,7 +722,7 @@ checkForOneAliasName decl  = fmap fst $ extractAlias decl False
 checkForOneCUName        :: CDecl -> Maybe Ident
 checkForOneCUName decl@(CDecl specs _ _)  =
   case [ts | CTypeSpec ts <- specs] of
-    [CSUType (CStruct _ n _ _ _) _] -> 
+    [CSUType (CStruct _ n _ _ _) _] ->
         case declaredDeclr decl of
           Nothing                       -> n
           Just (CDeclr _ [] _ _ _)      -> n -- no type derivations
@@ -724,9 +782,9 @@ lookupStructUnion ide preferTag useShadows = do
           then findObjShadow ide
           else liftM (fmap (\obj -> (obj, ide))) $ findObj ide
   let oobj = case mobj of
-        Just obj@(TypeCO _ , _) -> Just obj
-        Just obj@(BuiltinCO, _) -> Just obj
-        _                       -> Nothing
+        Just obj@(TypeCO{}, _)    -> Just obj
+        Just obj@(BuiltinCO{}, _) -> Just obj
+        _                         -> Nothing
   case preferTag of
     True -> case otag of
       Just tag -> extractStruct (posOf ide) tag
@@ -757,7 +815,7 @@ lookupStructUnion ide preferTag useShadows = do
 --
 lookupDeclOrTag                :: Ident -> Bool -> CT s (Either CDecl CTag)
 lookupDeclOrTag ide useShadows  = do
-  oobj <- findTypeObjMaybe ide useShadows
+  oobj <- findTypeObjMaybeWith True ide useShadows
   case oobj of
     Just (_, ide') -> liftM Left $ findAndChaseDecl ide' False False
                                                    -- already did check shadows
@@ -822,11 +880,12 @@ extractStruct pos (StructUnionCT su)  = do
                                     traceCTrav $ "def=" ++ show def ++ "\n"
                                     case def of
                                       TagCD tag -> extractStruct pos tag
-                                      bad_obj       -> err ide' bad_obj
+                                      UndefCD   -> incompleteTypeErr pos
+                                      bad_obj   -> err ide' bad_obj
     _                          -> return su
   where
     err ide bad_obj =
-      do interr $ "CTrav.extractStruct: Illegal reference! Expected " ++ dumpIdent ide ++ 
+      do interr $ "CTrav.extractStruct: Illegal reference! Expected " ++ dumpIdent ide ++
                   " to link to TagCD but refers to "++ (show bad_obj) ++ "\n"
 
 extractStruct' :: Position -> CTag -> CT s (Maybe CStructUnion)
@@ -912,3 +971,9 @@ structExpectedErr pos  =
   raiseErrorCTExc pos
     ["Expected a struct!",
      "Expected a structure or union; instead found an enum or basic type."]
+
+incompleteTypeErr     :: Position -> CT s a
+incompleteTypeErr pos  =
+  raiseErrorCTExc pos
+    ["Illegal use of incomplete type!",
+     "Expected a fully defined structure or union tag; instead found incomplete type."]
