@@ -77,7 +77,8 @@ module C2HS.Gen.Monad (
   objIs, queryObj, sizeIs, querySize, queryClass, queryPointer,
   mergeMaps, dumpMaps, queryEnum, isEnum,
   queryTypedef, isC2HSTypedef, queryDefaultMarsh, isDefaultMarsh,
-  addWrapper, getWrappers
+  addWrapper, getWrappers,
+  addHsDependency, getHsDependencies
 ) where
 
 -- standard libraries
@@ -249,6 +250,8 @@ instance Ord Wrapper where
 
 type WrapperSet = Set Wrapper
 
+type Dependencies = Set String
+
 {- FIXME: What a mess...
 instance Show HsObject where
   show (Pointer ptrType isNewtype) =
@@ -310,7 +313,8 @@ data GBState  = GBState {
   enums     :: EnumSet,              -- enumeration hooks
   tdmap     :: TypedefMap,           -- typedefs
   dmmap     :: DefaultMarshMap,      -- user-defined default marshallers
-  wrappers  :: WrapperSet
+  wrappers  :: WrapperSet,           -- C wrapper functions
+  deps      :: Dependencies          -- Haskell dependencies (for imports)
   }
 
 type GB a = CT GBState a
@@ -327,7 +331,8 @@ initialGBState  = GBState {
                     enums = Set.empty,
                     tdmap = Map.empty,
                     dmmap = Map.empty,
-                    wrappers = Set.empty
+                    wrappers = Set.empty,
+                    deps = Set.empty
                   }
 
 -- | set the dynamic library and library prefix
@@ -560,6 +565,14 @@ getWrappers :: GB [Wrapper]
 getWrappers = Set.toList `fmap` readCT wrappers
 
 
+-- | add Haskell module dependency for import generation
+addHsDependency :: String -> GB ()
+addHsDependency m = transCT (\st -> (st { deps = Set.insert m (deps st) }, ()))
+
+getHsDependencies :: GB [String]
+getHsDependencies = Set.toList `fmap` readCT deps
+
+
 -- error messages
 -- --------------
 
