@@ -7,6 +7,7 @@ import Test.HUnit hiding (Test, assert)
 import System.FilePath (searchPathSeparator)
 import System.Info (os)
 import Prelude hiding (FilePath)
+import Control.Monad (forM)
 import Control.Monad.IO.Class
 import Shelly
 import Data.List (sort)
@@ -80,6 +81,16 @@ tests =
     , testCase "Issue #123" issue123
     , testCase "Issue #127" issue127
     , testCase "Issue #128" issue128
+    , testCase "Issue #130" issue130
+    , testCase "Issue #131" issue131
+    , testCase "Issue #133" issue133
+    , testCase "Issue #134" issue134
+    , testCase "Issue #136" issue136
+    , testCase "Issue #140" issue140
+    , testCase "Issue #141" issue141
+    , testCase "Issue #149" issue149
+    , testCase "Issue #151" issue151
+    , testCase "Issue #152" issue152
     ] ++
     -- Some tests that won't work on Windows.
     if os /= "cygwin32" && os /= "mingw32"
@@ -100,6 +111,53 @@ call_capital = c2hsShelly $ chdir "tests/bugs/call_capital" $ do
   let expected = ["upper C();", "lower c();", "upper C();"]
   liftIO $ assertBool "" (T.lines res == expected)
 
+issue152 :: Assertion
+issue152 = hs_only_build_issue 152
+
+issue151 :: Assertion
+issue151 = hs_only_build_issue 151
+
+issue149 :: Assertion
+issue149 = build_issue_fails 149
+
+issue141 :: Assertion
+issue141 = c2hsShelly $ chdir "tests/bugs/issue-141" $ do
+  mapM_ rm_f ["Issue141A.hs", "Issue141A.chs.h", "Issue141A.chi",
+              "Issue141B.hs", "Issue141B.chs.h", "Issue141B.chi",
+              "Issue141C.hs", "Issue141C.chs.h", "Issue141C.chi"]
+  codes <- forM ["A", "B", "C"] $ \suff -> do
+    errExit False $ cmd "c2hs" $ "Issue141" <> suff <> ".chs"
+    lastExitCode
+  liftIO $ assertBool "" (all (/= 0) codes)
+
+issue140 :: Assertion
+issue140 = expect_issue 140 ["123", "456", "789"]
+
+issue136 :: Assertion
+issue136 = build_issue_tolerant 136
+
+issue134 :: Assertion
+issue134 = hs_only_build_issue 134
+
+issue133 :: Assertion
+issue133 = hs_only_build_issue 133
+
+issue131 :: Assertion
+issue131 = c2hsShelly $ chdir "tests/bugs/issue-131" $ do
+  mapM_ rm_f ["Issue131.hs", "Issue131.chs.h", "Issue131.chs.c", "Issue131.chi",
+              "issue131_c.o", "Issue131.chs.o", "Issue131"]
+  cmd "c2hs" "Issue131.chs"
+  cmd cc "-c" "-o" "issue131_c.o" "issue131.c"
+  cmd cc "-c" "Issue131.chs.c"
+  cmd "ghc" "--make" "issue131_c.o" "Issue131.chs.o" "Issue131.hs"
+  res <- absPath "./Issue131" >>= cmd
+  let expected = ["5", "3",
+                  "True", "False"]
+  liftIO $ assertBool "" (T.lines res == expected)
+
+issue130 :: Assertion
+issue130 = expect_issue 130  ["3", "3"]
+
 issue128 :: Assertion
 issue128 = c2hsShelly $ chdir "tests/bugs/issue-128" $ do
   mapM_ rm_f ["Issue128.hs", "Issue128.chs.h", "Issue128.chs.c", "Issue128.chi",
@@ -356,27 +414,30 @@ issue07 = c2hsShelly $ do
   code <- lastExitCode
   liftIO $ assertBool "" (code == 0)
 
-do_issue_build :: Bool -> Bool -> Int -> String -> [Text] -> Sh ()
-do_issue_build strict cbuild n ext c2hsargs =
+do_issue_build :: Bool -> Bool -> Int -> String -> String -> [Text] -> Sh ()
+do_issue_build strict cbuild n suff ext c2hsargs =
   let wdir = "tests/bugs" </> ("issue-" <> show n)
       lc = "issue" <> show n
       lcc = lc <> "_c"
-      uc = fromText $ T.pack $ "Issue" <> show n <>
+      uc = fromText $ T.pack $ "Issue" <> show n <> suff <>
            (if ext == "" then "" else "_" <> ext)
   in do
     cd wdir
     mapM_ rm_f [uc <.> "hs", uc <.> "chs.h", uc <.> "chi", lcc <.> "o", uc]
     run "c2hs" $ c2hsargs ++ [toTextIgnore $ uc <.> "chs"]
-    when cbuild $ cmd cc "-c" "-o" (lcc <.> "o") (lc <.> "c")
-    case (strict, cbuild) of
-      (True, True) ->
-        cmd "ghc" "-Wall" "-Werror" "--make" (lcc <.> "o") (uc <.> "hs")
-      (False, True) ->
-        cmd "ghc" "--make" (lcc <.> "o") (uc <.> "hs")
-      (True, False) ->
-        cmd "ghc" "-Wall" "-Werror" "--make" (uc <.> "hs")
-      (False, False) ->
-        cmd "ghc" "--make" (uc <.> "hs")
+    code <- lastExitCode
+    when (code == 0) $ do
+      when cbuild $ cmd cc "-c" "-o" (lcc <.> "o") (lc <.> "c")
+      code <- lastExitCode
+      when (code == 0) $ case (strict, cbuild) of
+        (True, True) ->
+          cmd "ghc" "-Wall" "-Werror" "--make" (lcc <.> "o") (uc <.> "hs")
+        (False, True) ->
+          cmd "ghc" "--make" (lcc <.> "o") (uc <.> "hs")
+        (True, False) ->
+          cmd "ghc" "-Wall" "-Werror" "--make" (uc <.> "hs")
+        (False, False) ->
+          cmd "ghc" "--make" (uc <.> "hs")
 
 expect_issue :: Int -> [Text] -> Assertion
 expect_issue n expected = expect_issue_with True True n "" [] expected
@@ -392,7 +453,7 @@ hs_only_expect_issue n ordered expected =
 expect_issue_with :: Bool -> Bool -> Int -> String -> [Text] -> [Text]
                   -> Assertion
 expect_issue_with ordered cbuild n ext c2hsargs expected = c2hsShelly $ do
-  do_issue_build True cbuild n ext c2hsargs
+  do_issue_build True cbuild n "" ext c2hsargs
   res <- absPath ("." </> (fromText $ T.pack $ "Issue" <> show n <>
                            (if ext == "" then "" else "_" <> ext))) >>= cmd
   liftIO $ assertBool "" $ case ordered of
@@ -401,7 +462,7 @@ expect_issue_with ordered cbuild n ext c2hsargs expected = c2hsShelly $ do
 
 build_issue_with :: Bool -> Bool -> Int -> [Text] -> Assertion
 build_issue_with strict cbuild n c2hsargs = c2hsShelly $ do
-  errExit False $ do_issue_build strict cbuild n "" c2hsargs
+  errExit False $ do_issue_build strict cbuild n "" "" c2hsargs
   code <- lastExitCode
   liftIO $ assertBool "" (code == 0)
 
@@ -413,3 +474,16 @@ build_issue_tolerant n = build_issue_with False True n []
 
 hs_only_build_issue :: Int -> Assertion
 hs_only_build_issue n = build_issue_with True False n []
+
+
+build_issue_fails_with :: Bool -> Bool -> Int -> [Text] -> Assertion
+build_issue_fails_with strict cbuild n c2hsargs = c2hsShelly $ do
+  errExit False $ do_issue_build strict cbuild n "" "" c2hsargs
+  code <- lastExitCode
+  liftIO $ assertBool "" (code /= 0)
+
+build_issue_fails :: Int -> Assertion
+build_issue_fails n = build_issue_fails_with True True n []
+
+hs_only_build_issue_fails :: Int -> Assertion
+hs_only_build_issue_fails n = build_issue_fails_with True False n []
